// rsspecan_sync_with_external_trigger_exampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "rsspecan_sync_with_external_trigger_example.h"
#include "rsspecan_sync_with_external_trigger_exampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define WINDOW        1


#define CHECKERR(fcal) if ((rsspecan_status = fcal) < 0)\
					   {\
							cleanup(VI_FALSE);\
							return;\
					   }

// Crsspecan_sync_with_external_trigger_exampleDlg dialog




Crsspecan_sync_with_external_trigger_exampleDlg::Crsspecan_sync_with_external_trigger_exampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(Crsspecan_sync_with_external_trigger_exampleDlg::IDD, pParent)
	, m_ResourceString(_T("GPIB::20::INSTR"))
	, m_IDQuery(TRUE)
	, m_Reset(TRUE)
	, m_Display(FALSE)
	, m_Timeout(5000)
	, m_State(_T("Ready"))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDI_RSICON);
}

void Crsspecan_sync_with_external_trigger_exampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_RESOURCE_STRING, m_ResourceString);
	DDX_Check(pDX, IDC_IDQUERY, m_IDQuery);
	DDX_Check(pDX, IDC_RESET, m_Reset);
	DDX_Check(pDX, IDC_DISPLAY, m_Display);
	DDX_Text(pDX, IDC_TIMEOUT, m_Timeout);
	DDX_Text(pDX, IDC_STATE, m_State);
}

BEGIN_MESSAGE_MAP(Crsspecan_sync_with_external_trigger_exampleDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_START, &Crsspecan_sync_with_external_trigger_exampleDlg::OnBnClickedStart)
END_MESSAGE_MAP()


// Crsspecan_sync_with_external_trigger_exampleDlg message handlers

BOOL Crsspecan_sync_with_external_trigger_exampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void Crsspecan_sync_with_external_trigger_exampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR Crsspecan_sync_with_external_trigger_exampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void Crsspecan_sync_with_external_trigger_exampleDlg::OnBnClickedStart()
{
	USES_CONVERSION;
	
	UpdateData ();

	m_State.Format(_T("Waiting for trigger"));
	UpdateData (false);

	HCURSOR cur = LoadCursor ( NULL, IDC_WAIT );
	SetCursor ( cur );


	/* initialize driver */
	CHECKERR (rsspecan_init ((char *) ((LPCTSTR) W2A(m_ResourceString.LockBuffer())), m_IDQuery, m_Reset, &m_InstrSession));
	m_ResourceString.UnlockBuffer();

	/* switch device display on/off */
	CHECKERR( rsspecan_ConfigureDisplayUpdate(m_InstrSession,m_Display));

	/* Set instrument to single sweep mode - continous mode is done
     * programmatically because this is the only way how to
     * synchronize measurement using *OPC
     */
    CHECKERR (rsspecan_ConfigureSignalStatisticSweep(m_InstrSession, VI_FALSE));

	/* configure external trigger */
    CHECKERR (rsspecan_ConfigureTriggerSource(m_InstrSession, WINDOW, 1));

    /* do a sweep */
    CHECKERR (rsspecan_Initiate(m_InstrSession, WINDOW, m_Timeout));

	m_State.Format(_T("Successfully finished"));
	UpdateData (false);

	cur = LoadCursor (NULL, IDC_ARROW);
	SetCursor (cur);
	
	/* close session */
	CHECKERR (rsspecan_close (m_InstrSession));
}

/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with error message.
/*           In case Device Clear is requested, it is necessary to set the
/*           argument deviceClear to VI_TRUE.
/*===========================================================================*/
void Crsspecan_sync_with_external_trigger_exampleDlg::cleanup(ViBoolean deviceClear)
{
	if (rsspecan_status < 0)
	{
		ViChar      error_message[256];
		ViStatus    temp_status;
		CString		message;
		CString		error_string;
		
		/* Query the error message from the instrument driver. This
		 * function gets the error message for VISA and driver failures
		 */
		temp_status = rsspecan_error_message(m_InstrSession, rsspecan_status, error_message);
		
		if (temp_status == VI_WARN_UNKNOWN_STATUS)
		{
			/* Query the error message from the instrument. This
			 * function gets the error message directly from instrument, like
			 * execution error, out of range error etc.
			 */
			rsspecan_error_query(m_InstrSession, &temp_status, error_message);
		}
		error_string = error_message;
		message.Format (_T ("Error 0x%8X occured:\n\n%s"), rsspecan_status, error_string);
		AfxMessageBox (message, MB_OK | MB_ICONSTOP); 
	}
	
	/* Device Clear is a low-level command and should be used in case the
	 * instrument is waiting for Operation Complete to cancel the wait.
	 * It is useful for instance in case of incorrect external trigger
	 * when the instrument does not respond to any other command because of
	 * waiting for trigger.
	 */
	if (deviceClear)
	{
		viClear (m_InstrSession);
		viPrintf (m_InstrSession, "*CLS\n");
	}
	
	rsspecan_close (m_InstrSession);
	
	m_State.Format(_T("Error occured"));
	UpdateData (false);

	HCURSOR cur = LoadCursor ( NULL, IDC_ARROW );
	SetCursor (cur);
}